@@ -32,6 +32,13 @@
 #include "event.hpp"
 #include "event_nonrt.hpp"
 
+//define timing for auto updates in ms
+//having STEP more often than 10ms and a different from the MIN may cause timing problems
+#define AUTO_UPDATE_MIN 10
+#define AUTO_UPDATE_STEP AUTO_UPDATE_MIN
+#define AUTO_UPDATE_MAX 100
+#define AUTO_UPDATE_RANGE (((AUTO_UPDATE_MAX - AUTO_UPDATE_MIN)/AUTO_UPDATE_STEP) + 1)
+
 namespace SooperLooper {
 
 class Engine;
@@ -58,7 +65,7 @@ class ControlOSC
 	
 	void send_all_midi_bindings (MidiBindings * bind, std::string returl, std::string retpath);
 
-	void send_auto_updates ();
+	void send_auto_updates (const std::list<short int> timeout_list);
 	void send_error (std::string returl, std::string retpath, std::string mesg);
 	
 	void finish_get_event (GetParamEvent & event);
@@ -134,6 +141,7 @@ class ControlOSC
 	static int _global_unregister_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
 	static int _global_register_auto_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
 	static int _global_unregister_auto_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
+
 	
 	static int _midi_start_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
 	static int _midi_stop_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
@@ -159,10 +167,12 @@ class ControlOSC
 	int save_session_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	int register_config_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	int unregister_config_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
+
 	int global_register_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	int global_unregister_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	int global_register_auto_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	int global_unregister_auto_update_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
+
 	
 	int midi_start_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	int midi_stop_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
@@ -209,22 +219,37 @@ class ControlOSC
 	typedef std::list<UrlPair> UrlList;
 	typedef std::map<InstancePair, UrlList > ControlRegistrationMap;
 	typedef std::map<InstancePair, float> LastValueMap;
+	typedef std::map<UrlPair, LastValueMap> UrlMapToLastValueMap;
+
+	typedef struct {
+		UrlPair upair;
+		short int timeout;
+	} UrlPairAuto;
+
+	typedef std::list<UrlPairAuto> UrlListAuto;
+	typedef std::map<InstancePair, UrlListAuto > ControlRegistrationMapAuto;
 	
 	ControlRegistrationMap _registration_map;
-	ControlRegistrationMap _auto_registration_map;
+	ControlRegistrationMapAuto _auto_registration_map;
 
-	LastValueMap     _last_value_map;
+	UrlMapToLastValueMap _last_value_map;
+
+	int compare_auto(UrlPairAuto a, UrlPairAuto b);
 
 	void send_registered_updates(std::string ctrl, float val, int instance, int source=-1);
 
 	bool send_registered_updates(ControlRegistrationMap::iterator & iter,
 				     std::string ctrl, float val, int instance, int source=-1);
+	bool send_registered_auto_updates(ControlRegistrationMapAuto::iterator & iter,
+				    const InstancePair & ipair, float val, const std::list<short int> timeout_list);
 	
 
 	
 	typedef std::pair<std::string, std::string> AddrPathPair;
 	typedef std::list<AddrPathPair> AddressList;
 	AddressList _config_registrations;
+
+	void validate_returl(std::string & returl);
 };
 
 };  // sooperlooper namespace
