@@ -20,6 +20,8 @@
 
 #include <cmath>
 #include <cstring>
+#include <fstream>
+#include <cstring>
 #include <unistd.h>
 #include <sys/time.h>
 #include <pthread.h>
@@ -38,7 +40,6 @@
 #include "midi_bind.hpp"
 #include "midi_bridge.hpp"
 #include "utils.hpp"
-#include "command_map.hpp"
 
 using namespace SooperLooper;
 using namespace std;
@@ -505,7 +506,7 @@ Engine::prepare_buffers(nframes_t nframes)
 		//	memset (outbuf, 0, nframes * sizeof(sample_t));
 		//}
 
-		memset (_temp_output_buffers[i], 0, nframes * sizeof(sample_t));		
+		memset (_temp_output_buffers[i], 0, nframes * sizeof(sample_t));
 		
 		// attenuate common inputs
 
@@ -660,7 +661,7 @@ Engine::get_osc_port ()
 
 
 static inline Event * next_rt_event (RingBuffer<Event>::rw_vector & vec, size_t & pos,
-				     RingBuffer<Event>::rw_vector & midivec, size_t & midipos)
+                                     RingBuffer<Event>::rw_vector & midivec, size_t & midipos)
 {
 	Event * e1 = 0;
 	Event * e2 = 0;
@@ -806,12 +807,23 @@ Engine::process (nframes_t nframes)
 			// handle special global RT events
 			if (evt->Instance == -2 
 			    || evt->Command == Event::SOLO
-			    || evt->Command == Event::SOLO_NEXT ||  evt->Command == Event::SOLO_PREV
-			    || evt->Command == Event::RECORD_SOLO_NEXT ||  evt->Command == Event::RECORD_SOLO_PREV 
+			    || evt->Command == Event::SOLO_NEXT
+			    || evt->Command == Event::SOLO_PREV
+			    || evt->Command == Event::RECORD_SOLO_NEXT
+			    || evt->Command == Event::RECORD_SOLO_PREV
 			    || evt->Command == Event::RECORD_SOLO
-				|| evt->Command == Event::RECORD_EXCLUSIVE_NEXT ||  evt->Command == Event::RECORD_EXCLUSIVE_PREV 
+			    || evt->Command == Event::RECORD_EXCLUSIVE_NEXT
+			    || evt->Command == Event::RECORD_EXCLUSIVE_PREV
 			    || evt->Command == Event::RECORD_EXCLUSIVE
-				|| evt->Command == Event::RECORD_OR_OVERDUB_EXCL_NEXT ||  evt->Command == Event::RECORD_OR_OVERDUB_EXCL_PREV || evt->Command == Event::RECORD_OR_OVERDUB_EXCL)
+			    || evt->Command == Event::RECORD_OR_OVERDUB_SOLO
+			    || evt->Command == Event::RECORD_OR_OVERDUB_SOLO_TRIG
+			    || evt->Command == Event::RECORD_OVERDUB_END_SOLO
+			    || evt->Command == Event::RECORD_OVERDUB_END_SOLO_TRIG
+			    || evt->Command == Event::RECORD_OR_OVERDUB_EXCL_NEXT
+			    || evt->Command == Event::RECORD_OR_OVERDUB_EXCL_PREV
+			    || evt->Command == Event::RECORD_OR_OVERDUB_EXCL
+			    || evt->Command == Event::RECORD_OR_OVERDUB_SOLO_NEXT
+			    || evt->Command == Event::RECORD_OR_OVERDUB_SOLO_PREV )
 			{
 				do_global_rt_event (evt, usedframes + doframes, nframes - (usedframes + doframes));
 				
@@ -867,6 +879,13 @@ Engine::process (nframes_t nframes)
 
 			usedframes += doframes;
 
+			// event is committed, if it is a control event, push it onto the nonrt update queue
+			if (evt->Type == Event::type_control_change || evt->Type == Event::type_global_control_change) {
+				do_push_control_event (_nonrt_update_event_queue, 
+				                       evt->Type, evt->Control, evt->Value, 
+				                       evt->Instance, evt->source);
+			}
+
 			evt = next_rt_event (vec, n, midivec, midi_n);
 		}
 
@@ -923,10 +942,23 @@ Engine::process (nframes_t nframes)
 void
 Engine::do_global_rt_event (Event * ev, nframes_t offset, nframes_t nframes)
 {
-	bool exclcmd = (ev->Command == Event::RECORD_EXCLUSIVE_NEXT ||  ev->Command == Event::RECORD_EXCLUSIVE_PREV || ev->Command == Event::RECORD_EXCLUSIVE);
-	bool exclocmd = (ev->Command == Event::RECORD_OR_OVERDUB_EXCL_NEXT ||  ev->Command == Event::RECORD_OR_OVERDUB_EXCL_PREV || ev->Command == Event::RECORD_OR_OVERDUB_EXCL); 
+	bool exclcmd         = (   ev->Command == Event::RECORD_EXCLUSIVE_NEXT 
+	                        || ev->Command == Event::RECORD_EXCLUSIVE_PREV 
+	                        || ev->Command == Event::RECORD_EXCLUSIVE);
+
+	bool exclocmd        = (   ev->Command == Event::RECORD_OR_OVERDUB_EXCL_NEXT 
+	                        || ev->Command == Event::RECORD_OR_OVERDUB_EXCL_PREV 
+	                        || ev->Command == Event::RECORD_OR_OVERDUB_EXCL); 
+
+	bool recOverSoloNext = (   ev->Command == Event::RECORD_OR_OVERDUB_SOLO_NEXT 
+	                        || ev->Command == Event::RECORD_OR_OVERDUB_SOLO_PREV);
+
+	bool recOverSolo     = (   ev->Command == Event::RECORD_OR_OVERDUB_SOLO 
+	                        || ev->Command == Event::RECORD_OR_OVERDUB_SOLO_TRIG 
+	                        || ev->Command == Event::RECORD_OVERDUB_END_SOLO 
+	                        || ev->Command == Event::RECORD_OVERDUB_END_SOLO_TRIG  
+	                        || recOverSoloNext);
 
-	
 	if (ev->Control == Event::TapTempo) {
 		nframes_t thisframe = _running_frames + offset;
 		if (thisframe > _last_tempo_frame) {
@@ -973,9 +1005,13 @@ Engine::do_global_rt_event (Event * ev, nframes_t offset, nframes_t nframes)
 		}
 		
 	}
-	else if (ev->Command == Event::SOLO || ev->Command == Event::SOLO_NEXT ||  ev->Command == Event::SOLO_PREV
-			 ||  ev->Command == Event::RECORD_SOLO_NEXT ||  ev->Command == Event::RECORD_SOLO_PREV || ev->Command == Event::RECORD_SOLO
-			 || exclcmd || exclocmd)
+	else if (ev->Command == Event::SOLO 
+	      || ev->Command == Event::SOLO_NEXT
+	      || ev->Command == Event::SOLO_PREV
+	      || ev->Command == Event::RECORD_SOLO_NEXT
+	      || ev->Command == Event::RECORD_SOLO_PREV
+	      || ev->Command == Event::RECORD_SOLO
+	      || recOverSolo || exclcmd || exclocmd)
 	{
 		// notify all loops they are being soloed or not (this acts as a toggle)
 		int target_instance = (ev->Instance == -3) ? _selected_loop : ev->Instance;
@@ -984,29 +1020,31 @@ Engine::do_global_rt_event (Event * ev, nframes_t offset, nframes_t nframes)
 		    || (ev->Type == Event::type_cmd_up && _running_frames > (_solo_down_stamp + _longpress_frames)))
 		{	
 			if (ev->Command == Event::SOLO_NEXT ||  ev->Command == Event::RECORD_SOLO_NEXT 
-				|| ev->Command == Event::RECORD_EXCLUSIVE_NEXT || ev->Command == Event::RECORD_OR_OVERDUB_EXCL_NEXT) {
+				|| ev->Command == Event::RECORD_EXCLUSIVE_NEXT || ev->Command == Event::RECORD_OR_OVERDUB_EXCL_NEXT || ev->Command == Event::RECORD_OR_OVERDUB_SOLO_NEXT) {
 				// increment selected
 				_selected_loop = (_selected_loop + 1) % _rt_instances.size();
 				_sel_loop_changed = true;
 				target_instance = _selected_loop;
 			}
 			else if (ev->Command == Event::SOLO_PREV ||  ev->Command == Event::RECORD_SOLO_PREV 
-					 || ev->Command == Event::RECORD_EXCLUSIVE_PREV || ev->Command == Event::RECORD_OR_OVERDUB_EXCL_PREV) {
+					 || ev->Command == Event::RECORD_EXCLUSIVE_PREV || ev->Command == Event::RECORD_OR_OVERDUB_EXCL_PREV || ev->Command == Event::RECORD_OR_OVERDUB_SOLO_PREV) {
 				// decrement selected
 				_selected_loop = _selected_loop > 0 ? _selected_loop - 1 : _rt_instances.size() - 1;
 				_sel_loop_changed = true;
 				target_instance = _selected_loop;
 			}
-			else if ( ev->Command == Event::RECORD_EXCLUSIVE || ev->Command == Event::RECORD_OR_OVERDUB_EXCL) {
+			else if ( ev->Command == Event::RECORD_EXCLUSIVE || ev->Command == Event::RECORD_OR_OVERDUB_EXCL || recOverSolo) {
 				// select the loop commanded in these cases
 				_selected_loop = target_instance;
 				_sel_loop_changed = true;
 			}
 		}
 
-
+                bool target_was_muted = false;
 		if (target_instance >= 0 && target_instance < (int) _rt_instances.size()) 
 		{
+			target_was_muted = _rt_instances[target_instance]->is_muted() &&  _rt_instances[target_instance]->has_loop(); 
+
 			if (ev->Type == Event::type_cmd_down || ev->Type == Event::type_cmd_hit || ev->Type == Event::type_cmd_upforce
 			    || (ev->Type == Event::type_cmd_up && _running_frames > (_solo_down_stamp + _longpress_frames)))
 			{
@@ -1019,16 +1057,21 @@ Engine::do_global_rt_event (Event * ev, nframes_t offset, nframes_t nframes)
 							// if it is in any active state, finish that state
 							(*i)->finish_state();
 						}
-					}										
+					}
 				}
 				else 
 				{
 					// solo commands
 					bool target_solo_state = _rt_instances[target_instance]->is_soloed();
-					
+					bool retrigger = (ev->Command == Event::RECORD_OR_OVERDUB_SOLO_TRIG || ev->Command == Event::RECORD_OVERDUB_END_SOLO_TRIG);
 					for (Instances::iterator i = _rt_instances.begin(); i != _rt_instances.end(); ++i) 
 					{
-						(*i)->set_soloed (target_instance, !target_solo_state);
+						if (recOverSolo) {
+						        // for this command we always want it to force solo on
+						        (*i)->set_soloed (target_instance, true, retrigger);
+						} else {
+						        (*i)->set_soloed (target_instance, !target_solo_state);
+						}
 					}
 				}
 				
@@ -1046,12 +1089,17 @@ Engine::do_global_rt_event (Event * ev, nframes_t offset, nframes_t nframes)
 			ev->Instance = target_instance;
 			ev->Command = Event::RECORD;
 		}
-		else if (exclocmd)
+		else if (exclocmd || (recOverSolo && (!target_was_muted || recOverSoloNext)))
 		{
 			// change the instance to the target we soloed, and the command to record
 			ev->Instance = target_instance;
-			ev->Command = Event::RECORD_OR_OVERDUB;
-			//cerr << "record or overdub exclusive" << endl;
+
+			if ((ev->Command == Event::RECORD_OVERDUB_END_SOLO || ev->Command == Event::RECORD_OVERDUB_END_SOLO_TRIG)) {
+			        ev->Command = Event::RECORD_OVERDUB_END_SOLO;
+			} else {
+			        ev->Command = Event::RECORD_OR_OVERDUB;
+			}
+			// cerr << "record or overdub exclusive: muted " <<  target_was_muted << endl;
 		}
 		else {
 			// change the event's instance so it isn't used later in the process call
@@ -1280,9 +1328,11 @@ Engine::push_control_event (Event::type_t type, Event::control_t ctrl, float val
 {
 	do_push_control_event (_event_queue, type, ctrl, val, instance);
 
+        // the nonrt update queue is now pushed on the realtime thread
+
 	// this is a known race condition, if the midi thread is changing controls
 	// simultaneously.  it's just an update :)
-	do_push_control_event (_nonrt_update_event_queue, type, ctrl, val, instance, src);
+	//do_push_control_event (_nonrt_update_event_queue, type, ctrl, val, instance, src);
 
 	// wakeup nonrt loop... this lock should really not block... but still
 	TentativeLockMonitor mon(_event_loop_lock,  __LINE__, __FILE__);
@@ -1295,9 +1345,11 @@ Engine::push_midi_control_event (Event::type_t type, Event::control_t ctrl, floa
 {
 	do_push_control_event (_midi_event_queue, type, ctrl, val, instance, framepos);
 
+        // the nonrt update queue is now pushed on the realtime thread
+
 	// this is a known race condition, if the osc thread is changing controls
 	// simultaneously.  it's just an update :)
-	do_push_control_event (_nonrt_update_event_queue, type, ctrl, val, instance);
+	//do_push_control_event (_nonrt_update_event_queue, type, ctrl, val, instance);
 
 	// wakeup nonrt loop... this lock should really not block... but still
 	TentativeLockMonitor mon(_event_loop_lock,  __LINE__, __FILE__);
@@ -1343,6 +1395,62 @@ Engine::push_sync_event (Event::control_t ctrl, long framepos, MIDI::timestamp_t
 	return;
 }
 
+//load functions are here rather than in midi_bindings.cpp for easy access to instance values
+//to initialize the toggle values.
+bool
+Engine::load_midi_bindings (string filename, bool append, CommandMap & cmdmap)
+{
+	//FILE * bindfile = 0;
+	std::ifstream bindfile;
+
+	bindfile.open(filename.c_str(), ios::in);
+	
+	if (!bindfile.is_open()) {
+		cerr << "sooperlooper warning: could not open for reading: " << filename << endl;
+		return false;
+	}
+	// todo: look for is in systemwide and ~/.sooperlooper/bindings/
+
+
+	return load_midi_bindings (bindfile, append, cmdmap);
+}
+
+bool
+Engine::load_midi_bindings (std::istream & instream, bool append, CommandMap & cmdmap)
+{
+	char  line[200];
+
+	
+	if (!append) {
+		_midi_bridge->bindings().clear_bindings();
+	}
+
+	while ( ! instream.eof())
+		//while (fgets (line, sizeof(line), bindfile) != NULL)
+		
+	{
+	        instream.getline (line, sizeof(line));
+		
+		// ignore empty lines and # lines
+		if (line[0] == '\n' || line[0] == '#' || line[0] == '\0') {
+			continue;
+		}
+
+		MidiBindInfo info;
+
+		if (!info.unserialize(line)) {
+			continue;
+		}
+
+		//initialize value for toggle with current value
+		if (info.style == MidiBindInfo::ToggleStyle) 
+			info.last_toggle_val = get_control_value(cmdmap.to_control_t(info.control), info.instance);
+
+		_midi_bridge->bindings().add_binding (info);
+	}
+
+	return true;
+}
 
 float
 Engine::get_control_value (Event::control_t ctrl, int8_t instance)
@@ -1422,12 +1530,18 @@ bool
 Engine::push_nonrt_event (EventNonRT * event)
 {
 
-	_nonrt_event_queue->write(&event, 1);
-	
-	LockMonitor mon(_event_loop_lock,  __LINE__, __FILE__);
-	pthread_cond_signal (&_event_cond);
+        if (_nonrt_event_queue->write_space() > 0) {
+                _nonrt_event_queue->write(&event, 1);
+                
+                LockMonitor mon(_event_loop_lock,  __LINE__, __FILE__);
+                pthread_cond_signal (&_event_cond);
 
-	return true;
+                return true;
+        }
+        else {
+                //cerr << "UGH, couldn't push event, no writespace" << endl;
+                return false;
+        }
 }
 
 	
@@ -1458,14 +1572,23 @@ Engine::mainloop()
 	struct timespec timeout;
 	struct timeval now = {0, 0};
 	struct timeval timeoutv = {0, 0};
+	struct timeval auto_update_timer_v[AUTO_UPDATE_RANGE];
+	struct timeval timer_last[AUTO_UPDATE_RANGE];
 	int  wait_ret = 0;
 	
 	EventNonRT * event;
 	Event  * evt;
 	LoopManageEvent * lmevt;
+
+	//initialize auto timeout arrays
+	for (int i = 0; i < AUTO_UPDATE_RANGE; i++) {
+		timer_last[i].tv_sec = 0;
+		timer_last[i].tv_usec = 0;
+		auto_update_timer_v[i].tv_sec = 0;
+		auto_update_timer_v[i].tv_usec = ((AUTO_UPDATE_STEP*(i+1)))*1000;
+	}
 	
 	// non-rt event processing loop
-
 	while (is_ok())
 	{
 		// pull off all loop management events from the rt thread
@@ -1516,7 +1639,7 @@ Engine::mainloop()
 				ParamChanged(evt->Control, instance); // emit
 			}
 			else if (evt->Type == Event::type_cmd_down || evt->Type == Event::type_cmd_hit) {
-				//cerr << "got command: " << evt->Command << endl;
+				cerr << "got nonrt- command: " << evt->Command << endl;
 				int instance = evt->Instance == -3 ? _selected_loop : evt->Instance;
 				ConfigUpdateEvent cuev (ConfigUpdateEvent::SendCmd, instance, evt->Command, "", "");
 				cuev.source = evt->source;
@@ -1598,21 +1721,33 @@ Engine::mainloop()
 
 		// if now is >= then the last timeout target, we should update
 		if (wait_ret == ETIMEDOUT || timercmp (&now, &timeoutv, >=)) {
-			//cerr << "timed out, sending updates" << endl;
-			_osc->send_auto_updates();
+			std::list<short int> timeout_list;
 
+			//work out for which auto timeouts it is time to update
+			for (short int i = 0; i < AUTO_UPDATE_RANGE; i++) {
+				struct timeval timer_diff = {0,0};
+				timersub(&now, &timer_last[i], &timer_diff);
+				if (timercmp(&timer_diff, &auto_update_timer_v[i], >=)) {
+					timeout_list.push_back((AUTO_UPDATE_STEP*(i+1)));
+					timer_last[i] = now;
+				}
+			}
+
+			_osc->send_auto_updates(timeout_list);
+			
 			// emit a parameter changed for state and others
 			for (unsigned int n=0; n < _instances.size(); ++n) {
 				ParamChanged(Event::State, n); // emit
+				ParamChanged(Event::Waiting, n);
 				ParamChanged(Event::LoopPosition, n);
 				ParamChanged(Event::LoopLength, n);
 				ParamChanged(Event::CycleLength, n);				
 				ParamChanged(Event::FreeTime, n);
 			}
 			
-			// wake up every 100 ms for servicing auto-update parameters
-			// TODO: make it more flexible
-			const long up_interval = 100000; // 100 ms
+			// wake up every 10 ms for servicing auto-update parameters
+			// TODO: make it more even more flexible?
+			const long up_interval = AUTO_UPDATE_STEP * 1000; // 10ms
 
 			timeout.tv_sec = now.tv_sec;
 			timeout.tv_nsec = (now.tv_usec + up_interval) * 1000;
@@ -1822,6 +1957,11 @@ Engine::process_nonrt_event (EventNonRT * event)
 			MidiBindInfo info;
 			if (info.unserialize (mb_event->bind_str)) {
 				bool exclus = (mb_event->options.find("exclusive") != string::npos);
+
+				//initialize value for toggle with current value
+				if (info.style == MidiBindInfo::ToggleStyle) 
+					info.last_toggle_val = get_control_value(cmdmap.to_control_t(info.control), info.instance);
+
 				LockMonitor lm (_midi_bridge->bindings_lock(), __LINE__, __FILE__);
 				_midi_bridge->bindings().add_binding (info, exclus);
 			}
@@ -1870,7 +2010,8 @@ Engine::process_nonrt_event (EventNonRT * event)
 		else if (mb_event->type == MidiBindingEvent::Load)
 		{
 			LockMonitor lm (_midi_bridge->bindings_lock(), __LINE__, __FILE__);
-			_midi_bridge->bindings().load_bindings (mb_event->bind_str, mb_event->options == "add" ? true: false);
+			load_midi_bindings(mb_event->bind_str, mb_event->options == "add" ? true: false, cmdmap);
+			//_midi_bridge->bindings().load_bindings (mb_event->bind_str, mb_event->options == "add" ? true: false);
 		}
 		else if (mb_event->type == MidiBindingEvent::Save)
 		{
@@ -1969,7 +2110,7 @@ Engine::set_tempo (double tempo, bool rt)
 	_tempo_counter = 0;
 
 
-	// adjust eigths per cycle if tempo is > 240 or < 60
+	// adjust eighths per cycle if tempo is > 240 or < 60
 	if (_smart_eighths &&
 	    _tempo > 1.0 && (_tempo > 240.0 || _tempo < 60.0)) {
 		//cerr << "tempo is " << _tempo << endl;
