@@ -155,6 +155,14 @@ MainPanel::~MainPanel()
 		// unregister
 		_loop_control->register_auto_updates((int) i, true);
 		_loop_control->register_input_controls((int) i, true);
+
+                // sleep for a little bit so the UDP isn't lost during this barrage
+#if wxCHECK_VERSION(2,5,3)
+                ::wxMilliSleep(50);
+#else
+                ::wxUsleep(50);
+#endif
+
 	}
 
 	delete _loop_control;
@@ -458,14 +466,7 @@ MainPanel::init_loopers (int count)
 	//_main_sizer->SetSizeHints( _scroller );   // set size hints to honour mininum size
 	
 	// request all values for initial state
-	_loop_control->request_global_values ();
-	
-	for (unsigned int i=0; i < _looper_panels.size(); ++i) {
-		_looper_panels[i]->set_index(i);
-		_loop_control->register_auto_updates((int) i);
-		_loop_control->register_input_controls((int) i);
-		_loop_control->request_all_values ((int)i);
-	}
+	_loop_control->register_all_in_new_thread(_looper_panels.size());
 
 	init_syncto_choice ();
 
@@ -1220,7 +1221,7 @@ void MainPanel::misc_action (bool release, wxString cmd)
 		}
 
 
-		wxString filename = do_file_selector (wxT("Choose file to save loop"), wxT("wav"), wxT("*.*"),  wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
+		wxString filename = do_file_selector (wxT("Choose file to save loop"), wxT("wav"), wxT("WAVE files (*.wav)|*.wav;*.WAV;*.Wav"),  wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
 		
 		if ( !filename.empty() )
 		{
@@ -1239,7 +1240,7 @@ void MainPanel::misc_action (bool release, wxString cmd)
 			index = 0;
 		}
 
-		wxString filename = do_file_selector (wxT("Choose file to open"), wxT(""), wxT("*.*"), wxOPEN|wxCHANGE_DIR);
+		wxString filename = do_file_selector (wxT("Choose file to open"), wxT(""), wxT("*.slsess"), wxOPEN|wxCHANGE_DIR);
 		
 		if ( !filename.empty() )
 		{
@@ -1351,7 +1352,7 @@ void MainPanel::set_curr_loop (int index)
 
 void MainPanel::do_load_session ()
 {
-	wxString filename = do_file_selector (wxT("Choose session to load"), wxT("*.slsess"), wxT("*.*"), wxOPEN|wxCHANGE_DIR);
+	wxString filename = do_file_selector (wxT("Choose session to load"), wxT("*.slsess"), wxT("*.slsess"), wxOPEN|wxCHANGE_DIR);
 	
 	if ( !filename.empty() )
 	{
@@ -1362,7 +1363,7 @@ void MainPanel::do_load_session ()
 
 void MainPanel::do_save_session (bool write_audio)
 {
-	wxString filename = do_file_selector (wxT("Choose file to save session"), wxT("slsess"), wxT("*.*"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
+	wxString filename = do_file_selector (wxT("Choose file to save session"), wxT("slsess"), wxT("*.slsess"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
 	
 	if ( !filename.empty() )
 	{
@@ -1377,23 +1378,32 @@ void MainPanel::do_save_session (bool write_audio)
 
 wxString MainPanel::do_file_selector(const wxString & message, const wxString & ext, const wxString & wc, int style)
 {
-	wxString filename;
+	wxString filestring;
 
 	_keyboard->set_enabled(false);
 	
 	if (_loop_control->is_engine_local()) {
-		filename = ::wxFileSelector(message, wxT(""), wxT(""), ext, wc, style);
+		wxFileName filename;
+		if (_last_used_path.IsEmpty()) {
+			_last_used_path = filename.GetHomeDir();
+		}
+		filestring = ::wxFileSelector(message, _last_used_path, wxT(""), ext, wc, style);
+		if (!filestring.IsEmpty()) {
+				filename.Assign(filestring);
+				_last_used_path = filename.GetPath();
+		}
 	}
 	else {
 		// popup basic filename text entry
-		filename = ::wxGetTextFromUser(wxString::Format(wxT("%s on remote host '%s'"), message.c_str(),
+		filestring = ::wxGetTextFromUser(wxString::Format(wxT("%s on remote host '%s'"), message.c_str(),
 								_loop_control->get_engine_host().c_str())
 					       , message);
 	}
+	//cerr << "last used:" << _last_used_path.ToAscii() << endl;
 
 	_keyboard->set_enabled(true);
 
-	return filename;
+	return filestring;
 }
 
 
@@ -1436,7 +1446,7 @@ AddCustomLoopDialog::AddCustomLoopDialog (MainPanel * parent, wxWindowID id, con
 	
 	statText = new wxStaticText(this, -1, wxT("Loop time (secs minimum):"));
 	setsizer->Add (statText, 0, wxALIGN_RIGHT|wxALIGN_CENTRE_VERTICAL);
-	_secs_per_channel_spin = new wxSpinCtrl(this, -1, wxT("20"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 4, 1000, (int)secs_channel);
+	_secs_per_channel_spin = new wxSpinCtrl(this, -1, wxT("20"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 4, 1000000, (int)secs_channel);
 	_secs_per_channel_spin->SetValue ((int)secs_channel);
 	setsizer->Add (_secs_per_channel_spin, 0);
 
