@@ -26,10 +26,9 @@
 #include <vector>
 #include <list>
 
-#include "gui_frame.hpp"
+#include "main_panel.hpp"
 #include "midi_bind_panel.hpp"
 #include "loop_control.hpp"
-#include "gui_app.hpp"
 #include "keyboard_target.hpp"
 
 #include <midi_bind.hpp>
@@ -58,7 +57,10 @@ enum {
 	ID_LoadButton,
 	ID_SaveButton,
 	ID_SusCheck,
-	ID_AppendCheck
+	ID_AppendCheck,
+	ID_DataMinCtrl,
+	ID_DataMaxCtrl,
+
 
 };
 
@@ -82,9 +84,16 @@ BEGIN_EVENT_TABLE(SooperLooperGui::MidiBindPanel, wxPanel)
 	
 END_EVENT_TABLE()
 
-static const wxString CcString("CC");
-static const wxString NoteString("Note");
-static const wxString PcString("PC");
+static const wxString CcString(wxT("CC"));
+static const wxString CcOnString(wxT("CC On"));
+static const wxString CcOffString(wxT("CC Off"));
+static const wxString NoteString(wxT("Note"));
+static const wxString NoteOnString(wxT("Note On"));
+static const wxString NoteOffString(wxT("Note Off"));
+static const wxString PcString(wxT("PC"));
+static const wxString PitchBendString(wxT("Pitch Bend"));
+static const wxString KeyPressureString(wxT("Key Pressure"));
+static const wxString ChannelPressureString(wxT("Channel Pressure"));
 	
 static int wxCALLBACK list_sort_callback (long item1, long item2, long sortData)
 {
@@ -97,13 +106,13 @@ static int wxCALLBACK list_sort_callback (long item1, long item2, long sortData)
 
 	
 // ctor(s)
-MidiBindPanel::MidiBindPanel(GuiFrame * guiframe, wxWindow * parent, wxWindowID id,
+MidiBindPanel::MidiBindPanel(MainPanel * mainpan, wxWindow * parent, wxWindowID id,
 		       const wxPoint& pos,
 		       const wxSize& size,
 		       long style ,
 		       const wxString& name)
 
-	: wxPanel ((wxWindow *)parent, id, pos, size, style, name), _parent (guiframe)
+	: wxPanel ((wxWindow *)parent, id, pos, size, style, name), _parent (mainpan)
 {
 	_learning = false;
 	_justResized = false;
@@ -196,21 +205,28 @@ void MidiBindPanel::init()
 	populate_controls();
 	//_control_combo->SetSelection(0);
 	colsizer->Add (_control_combo, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 2);
+	_control_combo->SetWindowVariant(wxWINDOW_VARIANT_NORMAL);
 
 	rowsizer = new wxBoxSizer(wxHORIZONTAL);
 	staticText = new wxStaticText(_edit_panel, -1, wxT("Loop #"), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (staticText, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 
 	_loopnum_combo =  new wxChoice(_edit_panel, ID_LoopNumCombo, wxDefaultPosition, wxSize(100, -1), 0, 0);
+	_loopnum_combo->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+	_loopnum_combo->Append (wxT("Selected"), (void *) -2);
 	_loopnum_combo->Append (wxT("Global"), (void *) -1);
 	_loopnum_combo->Append (wxT("All"), (void *) 0);
+
 	for (int i=1; i <= 16; ++i) {
 		_loopnum_combo->Append (wxString::Format(wxT("%d"), i), (void *) i);
 	}
-	_loopnum_combo->SetSelection(1);
+	_loopnum_combo->SetSelection(2);
 	rowsizer->Add (_loopnum_combo, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
 
 	_sus_check = new wxCheckBox(_edit_panel, ID_SusCheck, wxT("SUS"));
+	_sus_check->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+
 	rowsizer->Add (_sus_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
 	
 	colsizer->Add (rowsizer, 0, wxALL|wxEXPAND, 0);
@@ -228,44 +244,64 @@ void MidiBindPanel::init()
 
 	rowsizer = new wxBoxSizer(wxHORIZONTAL);
 	staticText = new wxStaticText(_edit_panel, -1, wxT("Ch#"), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+
 	rowsizer->Add (staticText, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 
 	_chan_spin =  new wxSpinCtrl(_edit_panel, ID_ChanSpin, wxT("1"), wxDefaultPosition, wxSize(50,-1), wxSP_ARROW_KEYS, 1, 16, 1, wxT("KeyAware"));
+	_chan_spin->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (_chan_spin, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 	
 	_type_combo = new wxChoice(_edit_panel, ID_TypeCombo,  wxDefaultPosition, wxSize(100, -1), 0, 0);
+	_type_combo->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	//_control_combo->SetToolTip(wxT("Choose control or command"));
 	_type_combo->Append (NoteString);
+	_type_combo->Append (NoteOnString);
+	_type_combo->Append (NoteOffString);
 	_type_combo->Append (CcString);
+	_type_combo->Append (CcOnString);
+	_type_combo->Append (CcOffString);
 	_type_combo->Append (PcString);
+	_type_combo->Append (KeyPressureString);
+	_type_combo->Append (ChannelPressureString);
+	_type_combo->Append (PitchBendString);
 	_type_combo->SetSelection(0);
-	rowsizer->Add (_type_combo, 1, wxALL|wxALIGN_CENTRE_VERTICAL|wxEXPAND, 2);
+	rowsizer->Add (_type_combo, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 
 	_param_spin =  new wxSpinCtrl(_edit_panel, ID_ParamSpin, wxT("0"), wxDefaultPosition, wxSize(50,-1), wxSP_ARROW_KEYS, 0, 127, 0, wxT("KeyAware"));
+	_param_spin->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (_param_spin, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 
 	_learn_button = new wxButton (_edit_panel, ID_LearnButton, wxT("Learn"), wxDefaultPosition, wxSize(-1, -1));
+	_learn_button->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (_learn_button, 0, wxALL, 3);
 
 	colsizer->Add (rowsizer, 0, wxALL|wxEXPAND, 0);
 
 	_range_panel = new wxPanel(_edit_panel);
 	rowsizer = new wxBoxSizer(wxHORIZONTAL);
-	staticText = new wxStaticText(_range_panel, -1, wxT("Range"), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText = new wxStaticText(_range_panel, -1, wxT("Targ Range"), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (staticText, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 
 	_lbound_ctrl = new wxTextCtrl(_range_panel, ID_LBoundCtrl, wxT(""), wxDefaultPosition, wxSize(40, -1), 0, wxDefaultValidator, wxT("KeyAware"));
+	_lbound_ctrl->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (_lbound_ctrl, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
 
 	staticText = new wxStaticText(_range_panel, -1, wxT(" to "), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (staticText, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
 
 	_ubound_ctrl = new wxTextCtrl(_range_panel, ID_UBoundCtrl, wxT(""), wxDefaultPosition, wxSize(40, -1), 0, wxDefaultValidator, wxT("KeyAware"));
+	_ubound_ctrl->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	rowsizer->Add (_ubound_ctrl, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
 
 	_style_combo =  new wxChoice(_range_panel, ID_StyleCombo,  wxDefaultPosition, wxSize(30, -1), 0, 0);
+	_style_combo->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 	_style_combo->Append (wxT("Normal"), (void *) MidiBindInfo::NormalStyle);
 	_style_combo->Append (wxT("Gain"), (void *) MidiBindInfo::GainStyle);
+	_style_combo->Append (wxT("Integer"), (void *) MidiBindInfo::IntegerStyle);
+	_style_combo->Append (wxT("Toggle"), (void *) MidiBindInfo::ToggleStyle);
 	_style_combo->SetToolTip(wxT("Choose a scaling type"));
 	_style_combo->SetSelection(0);
 	rowsizer->Add (_style_combo, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
@@ -277,6 +313,32 @@ void MidiBindPanel::init()
 	
 	colsizer->Add (_range_panel, 0, wxALL|wxEXPAND, 0);
 
+
+	_data_range_panel = new wxPanel(_edit_panel);
+	rowsizer = new wxBoxSizer(wxHORIZONTAL);
+	staticText = new wxStaticText(_data_range_panel, -1, wxT("Data Range"), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+	rowsizer->Add (staticText, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
+
+	_data_min_ctrl = new wxTextCtrl(_data_range_panel, ID_DataMinCtrl, wxT(""), wxDefaultPosition, wxSize(40, -1), 0, wxDefaultValidator, wxT("KeyAware"));
+	_data_min_ctrl->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+	rowsizer->Add (_data_min_ctrl, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
+
+	staticText = new wxStaticText(_data_range_panel, -1, wxT(" to "), wxDefaultPosition, wxSize(-1, -1), wxALIGN_LEFT);
+	staticText->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+	rowsizer->Add (staticText, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
+
+	_data_max_ctrl = new wxTextCtrl(_data_range_panel, ID_DataMaxCtrl, wxT(""), wxDefaultPosition, wxSize(40, -1), 0, wxDefaultValidator, wxT("KeyAware"));
+	_data_max_ctrl->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+	rowsizer->Add (_data_max_ctrl, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 1);
+
+	_data_range_panel->SetAutoLayout( true );     // tell dialog to use sizer
+        _data_range_panel->SetSizer( rowsizer );      // actually set the sizer
+	//rowsizer->SetSizeHints( _range_panel );   // set size hints to honour mininum size
+	//rowsizer->Fit(_data_range_panel);
+	
+	colsizer->Add (_data_range_panel, 0, wxALL|wxEXPAND, 0);
+
 	
 	editsizer->Add (colsizer, 1, wxALL|wxEXPAND, 4);
 
@@ -290,6 +352,7 @@ void MidiBindPanel::init()
 	
 	topsizer->Add (_edit_panel, 0, wxEXPAND|wxALL, 1);
 
+
 	buttsizer = new wxBoxSizer(wxHORIZONTAL);
 	buttsizer->Add (1,1, 1, wxALL, 0);
 
@@ -315,7 +378,6 @@ void MidiBindPanel::init()
 	topsizer->Add (buttsizer, 0, wxALL|wxEXPAND|wxALIGN_RIGHT, 4);
 
 
-
 	_parent->get_loop_control().MidiBindingChanged.connect (slot (*this, &MidiBindPanel::got_binding_changed));
 	_parent->get_loop_control().ReceivedNextMidi.connect (slot (*this, &MidiBindPanel::recvd_next_midi));
 	_parent->get_loop_control().NextMidiCancelled.connect (slot (*this, &MidiBindPanel::cancelled_next_midi));
@@ -334,18 +396,27 @@ void MidiBindPanel::populate_controls()
 	CommandMap & cmap = CommandMap::instance();
 	
 	cmap.get_commands(_cmdlist);
-	cmap.get_controls(_cmdlist);
+	cmap.get_controls(_ctrlist);
+
+	_cmdlist.sort();
+	_ctrlist.sort();
 
 	for (list<string>::iterator iter = _cmdlist.begin(); iter != _cmdlist.end(); ++iter)
 	{
 		if (cmap.is_command(*iter)) {
-			_control_combo->Append (wxString::Format("[cmd]  %s", iter->c_str()), (void *) (iter->c_str()));
+			_control_combo->Append (wxString::Format(wxT("[cmd]  %s"), wxString::FromAscii(iter->c_str()).c_str()), (void *) (iter->c_str()));
 		}
-		else if (cmap.is_input_control(*iter)) {
-			_control_combo->Append (wxString::Format("[ctrl]  %s", iter->c_str()), (void *) (iter->c_str()));
+	}
+	for (list<string>::iterator iter = _ctrlist.begin(); iter != _ctrlist.end(); ++iter)
+	{
+		if (cmap.is_input_control(*iter)) {
+			_control_combo->Append (wxString::Format(wxT("[ctrl]  %s"), wxString::FromAscii(iter->c_str()).c_str()), (void *) (iter->c_str()));
 		}
-		else if (cmap.is_global_control(*iter)) {
-			_control_combo->Append (wxString::Format("[g. ctrl]  %s", iter->c_str()), (void *) (iter->c_str()));
+	}
+	for (list<string>::iterator iter = _ctrlist.begin(); iter != _ctrlist.end(); ++iter)
+	{
+		if (cmap.is_global_control(*iter)) {
+			_control_combo->Append (wxString::Format(wxT("[g. ctrl]  %s"), wxString::FromAscii(iter->c_str()).c_str()), (void *) (iter->c_str()));
 		}
 	}
 	
@@ -429,17 +500,24 @@ void MidiBindPanel::refresh_state()
 		else if (info.instance == -2) {
 			item.SetText (wxT("Global"));
 		}
+		else if (info.instance == -3) {
+			item.SetText (wxT("Selected"));
+		}
 		_listctrl->SetItem (item);
 
 		// midi event
 		item.SetColumn(2);
-		item.SetText (wxString::Format(wxT("ch%d - %s - %d"), info.channel+1, info.type.c_str(), info.param));
+		item.SetText (wxString::Format(wxT("ch%d - %s - %d"), info.channel+1, wxString::FromAscii(info.type.c_str()).c_str(), info.param));
 		_listctrl->SetItem (item);
 
 		// range
 		item.SetColumn(3);
 		if (info.command == "set") {
-			item.SetText (wxString::Format(wxT("%g - %g %s"), info.lbound, info.ubound, info.style == MidiBindInfo::GainStyle ? wxT("gain") : wxT("")));
+			item.SetText (wxString::Format(wxT("%g - %g  (%d - %d)  %s"), info.lbound, info.ubound,
+						       info.data_min, info.data_max, 
+						       info.style == MidiBindInfo::GainStyle ? wxT("gain") : 
+						       ( info.style == MidiBindInfo::ToggleStyle ? wxT("togg") : 
+							 ( info.style == MidiBindInfo::IntegerStyle ? wxT("int") :  wxT("") ))));
 		}
 		else {
 			if (info.command == "susnote") {
@@ -455,7 +533,7 @@ void MidiBindPanel::refresh_state()
 		itemid++;
 	}
 
-	_listctrl->SortItems (list_sort_callback, (unsigned) _listctrl);
+	_listctrl->SortItems (list_sort_callback, (unsigned long) _listctrl);
 	
 	for (long i=0; i < _listctrl->GetItemCount(); ++i) {
 		item.SetId(i);
@@ -472,7 +550,6 @@ void MidiBindPanel::refresh_state()
 		}
 	}
 	
-
 // 	if (selexists) {
 // 		_edit_panel->Enable(true);
 // 	} else {
@@ -488,67 +565,109 @@ void MidiBindPanel::update_entry_area(MidiBindInfo * usethis)
 		return;
 	}
 
-	MidiBindInfo * info = static_cast<MidiBindInfo *>((void *)_listctrl->GetItemData(_selitem));
+	MidiBindInfo * info = 0;
 
 	if (usethis) {
 		info = usethis;
 	}
+	else if (_selitem >=0) {
+		info = static_cast<MidiBindInfo *>((void *)_listctrl->GetItemData(_selitem));
+	}
 
 	if (!info) {
 		return;
 	}
 
-	for (int i=0; i < _control_combo->GetCount(); ++i) {
+	for (int i=0; i < (int)_control_combo->GetCount(); ++i) {
 		if (static_cast<const char *>(_control_combo->GetClientData(i)) == info->control) {
 			_control_combo->SetSelection(i);
 			break;
 		}
 	}
 
-	_loopnum_combo->SetSelection(info->instance + 2);
+	_loopnum_combo->SetSelection(info->instance + 3);
 	_chan_spin->SetValue(info->channel + 1);
 
 	if (info->type == "cc") {
 		_type_combo->SetStringSelection(CcString);
 	}
+	else if (info->type == "ccoff") {
+		_type_combo->SetStringSelection(CcOffString);
+	}
+	else if (info->type == "ccon") {
+		_type_combo->SetStringSelection(CcOnString);
+	}
 	else if (info->type == "n") {
 		_type_combo->SetStringSelection(NoteString);
 	}
+	else if (info->type == "on") {
+		_type_combo->SetStringSelection(NoteOnString);
+	}
+	else if (info->type == "off") {
+		_type_combo->SetStringSelection(NoteOffString);
+	}
 	else if (info->type == "pc") {
 		_type_combo->SetStringSelection(PcString);
 	}
+	else if (info->type == "pb") {
+		_type_combo->SetStringSelection(PitchBendString);
+	}
+	else if (info->type == "kp") {
+		_type_combo->SetStringSelection(KeyPressureString);
+	}
+	else if (info->type == "cp") {
+		_type_combo->SetStringSelection(ChannelPressureString);
+	}
 
 	if (info->command == "set") {
 		_range_panel->Enable(true);
+		_data_range_panel->Enable(true);
+
+		_lbound_ctrl->SetValue (wxString::Format(wxT("%g"), info->lbound));
+		_ubound_ctrl->SetValue (wxString::Format(wxT("%g"), info->ubound));
+		
+		_data_min_ctrl->SetValue (wxString::Format(wxT("%d"), info->data_min));
+		_data_max_ctrl->SetValue (wxString::Format(wxT("%d"), info->data_max));
 	}
 	else {
-		_range_panel->Enable(false);
+		_range_panel->Enable(false); 
+		_data_range_panel->Enable(false); 
 		if (info->command == "susnote") {
 			_sus_check->SetValue(true);
 		}
 		else {
 			_sus_check->SetValue(false);
 		}
+
+		_lbound_ctrl->SetValue (wxT(""));
+		_ubound_ctrl->SetValue (wxT(""));
+		
+		_data_min_ctrl->SetValue (wxT(""));
+		_data_max_ctrl->SetValue (wxT(""));
 	}
 	
 	_param_spin->SetValue(info->param);
 
 
-	_lbound_ctrl->SetValue (wxString::Format("%g", info->lbound));
-	_ubound_ctrl->SetValue (wxString::Format("%g", info->ubound));
-
-	if (info->style == MidiBindInfo::NormalStyle) {
-		_style_combo->SetSelection(0);
+	if (info->style == MidiBindInfo::GainStyle) {
+		_style_combo->SetSelection(1);
+	}
+	else if (info->style == MidiBindInfo::ToggleStyle) {
+		_style_combo->SetSelection(3);
+	}
+	else if (info->style == MidiBindInfo::IntegerStyle) {
+		_style_combo->SetSelection(2);
 	}
 	else {
-		_style_combo->SetSelection(1);
+		_style_combo->SetSelection(0);
 	}
 	
 }
 
 void MidiBindPanel::update_curr_binding()
 {
-	double fval;
+	double fval = 0.0;
+	long  lval = 0;
 	CommandMap & cmap = CommandMap::instance();
 
 	if (_control_combo->GetSelection() < 0) {
@@ -560,7 +679,7 @@ void MidiBindPanel::update_curr_binding()
 	_currinfo.channel = _chan_spin->GetValue() - 1;
 
 	if (_loopnum_combo->GetSelection() >= 0) {
-		_currinfo.instance = (int) _loopnum_combo->GetClientData(_loopnum_combo->GetSelection()) - 1;
+		_currinfo.instance = (int) (long) _loopnum_combo->GetClientData(_loopnum_combo->GetSelection()) - 1;
 	}
 	else {
 		_currinfo.instance = -1;
@@ -571,15 +690,36 @@ void MidiBindPanel::update_curr_binding()
 	if (tsel == NoteString) {
 		_currinfo.type = "n";
 	}
+	else if (tsel == NoteOnString) {
+		_currinfo.type = "on";
+	}
+	else if (tsel == NoteOffString) {
+		_currinfo.type = "off";
+	}
 	else if (tsel == CcString) {
 		_currinfo.type = "cc";
 	}
+	else if (tsel == CcOnString) {
+		_currinfo.type = "ccon";
+	}
+	else if (tsel == CcOffString) {
+		_currinfo.type = "ccoff";
+	}
 	else if (tsel == PcString) {
 		_currinfo.type = "pc";
 	}
+	else if (tsel == PitchBendString) {
+		_currinfo.type = "pb";
+	}
+	else if (tsel == KeyPressureString) {
+		_currinfo.type = "kp";
+	}
+	else if (tsel == ChannelPressureString) {
+		_currinfo.type = "cp";
+	}
 
 	if (cmap.is_command(_currinfo.control)) {
-		if (_currinfo.type == "pc") {
+		if (_currinfo.type == "pc" || _currinfo.type == "cc") {
 			_currinfo.command = "hit";
 		}
 		else {
@@ -603,6 +743,10 @@ void MidiBindPanel::update_curr_binding()
 	
 	_currinfo.param = _param_spin->GetValue();
 
+	if (tsel == PitchBendString || tsel == ChannelPressureString) {
+		_currinfo.param = 0;
+	}
+
 	if (_lbound_ctrl->GetValue().ToDouble(&fval)) {
 		_currinfo.lbound = (float) fval;
 	}
@@ -617,9 +761,34 @@ void MidiBindPanel::update_curr_binding()
 		_currinfo.ubound = 1.0;
 	}
 
+	if (_data_min_ctrl->GetValue().ToLong(&lval)) {
+		//cerr << "data minis: " << (const char *) _data_min_ctrl->GetValue().ToAscii() << endl;
+		_currinfo.data_min = (int) lval;
+	}
+	else {
+		_currinfo.data_min = 0;
+	}
+
+	if (_data_max_ctrl->GetValue().ToLong(&lval)) {
+		//cerr << "data max is: " << (const char *) _data_max_ctrl->GetValue().ToAscii() << endl;
+		_currinfo.data_max = (int) lval;
+	}
+	else if (tsel == PitchBendString) {
+		_currinfo.data_max = 16383;
+	}
+	else {
+		_currinfo.data_max = 127;
+	}
+
 	if (_style_combo->GetSelection() == 1) {
 		_currinfo.style = MidiBindInfo::GainStyle;
 	}
+	else if (_style_combo->GetSelection() == 2) {
+		_currinfo.style = MidiBindInfo::IntegerStyle;
+	}
+	else if (_style_combo->GetSelection() == 3) {
+		_currinfo.style = MidiBindInfo::ToggleStyle;
+	}
 	else {
 		_currinfo.style = MidiBindInfo::NormalStyle;
 	}
@@ -636,9 +805,23 @@ void MidiBindPanel::on_combo (wxCommandEvent &ev)
 
  	if (CommandMap::instance().is_command(control)) {
  		_range_panel->Enable(false);
+ 		_data_range_panel->Enable(false);
+		_lbound_ctrl->SetValue(wxT(""));
+		_ubound_ctrl->SetValue(wxT(""));
+		_data_min_ctrl->SetValue(wxT(""));
+		_data_max_ctrl->SetValue(wxT(""));
  	}
  	else {
  		_range_panel->Enable(true);
+ 		_data_range_panel->Enable(true);
+
+		// set default ranges
+		CommandMap::ControlInfo info;
+		CommandMap::instance().get_control_info(control, info);
+		_lbound_ctrl->SetValue(wxString::Format(wxT("%g"), info.minValue));
+		_ubound_ctrl->SetValue(wxString::Format(wxT("%g"), info.maxValue));
+		_data_min_ctrl->SetValue (wxT(""));
+		_data_max_ctrl->SetValue (wxT(""));
  	}
 }
 
@@ -697,63 +880,23 @@ void MidiBindPanel::on_button (wxCommandEvent &ev)
 	}
 	else if (ev.GetId() == ID_LoadButton)
 	{
-		if (_parent->get_loop_control().is_engine_local()) {
-			
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(false);
-			wxString filename = wxFileSelector(wxT("Choose midi binding file to open"), wxT(""), wxT(""), wxT(""), wxT("*.slb"), wxOPEN|wxCHANGE_DIR);
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(true);
-
-			if ( !filename.empty() )
-			{
-				_parent->get_loop_control().load_midi_bindings(filename,  _append_check->GetValue());
-			}
+		wxString filename = _parent->do_file_selector(wxT("Choose midi binding file to open"), wxT(""), wxT("*.slb"), wxOPEN|wxCHANGE_DIR);
+		if ( !filename.empty() )
+		{
+			_parent->get_loop_control().load_midi_bindings(filename,  _append_check->GetValue());
 		}
-		else {
-			// popup basic filename text entry
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(false);
-
-			wxString filename = ::wxGetTextFromUser(wxString::Format("Specify midi binding file to load on remote host '%s'",
-										 _parent->get_loop_control().get_engine_host().c_str())
-								, wxT("Load Midi Binding"));
-
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(true);
-
-			if (!filename.empty()) {
-				_parent->get_loop_control().load_midi_bindings(filename, _append_check->GetValue());
-			}
-		}
-			
 	}
 	else if (ev.GetId() == ID_SaveButton)
 	{
-		if (_parent->get_loop_control().is_engine_local()) {
-			
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(false);
-			wxString filename = wxFileSelector(wxT("Choose midi binding file to save"), wxT(""), wxT(""), wxT(""), wxT("*.slb"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(true);
+		wxString filename = _parent->do_file_selector(wxT("Choose midi binding file to save"), wxT(""), wxT("*.slb"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
 
-			if ( !filename.empty() )
-			{
-				// add .slb if there isn't one already
-				if (filename.size() > 4 && filename.substr(filename.size() - 4, 4) != wxT(".slb")) {
-					filename += wxT(".slb");
-				}
-				_parent->get_loop_control().save_midi_bindings(filename);
-			}
-		}
-		else {
-			// popup basic filename text entry
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(false);
-
-			wxString filename = ::wxGetTextFromUser(wxString::Format("Specify midi binding file to save on remote host '%s'",
-										 _parent->get_loop_control().get_engine_host().c_str())
-								, wxT("Save Midi Binding"));
-
-			::wxGetApp().getFrame()->get_keyboard().set_enabled(true);
-
-			if (!filename.empty()) {
-				_parent->get_loop_control().save_midi_bindings(filename);
+		if ( !filename.empty() )
+		{
+			// add .slb if there isn't one already
+			if (filename.size() > 4 && filename.substr(filename.size() - 4, 4) != wxT(".slb")) {
+				filename += wxT(".slb");
 			}
+			_parent->get_loop_control().save_midi_bindings(filename);
 		}
 	}
 	else {
