@@ -94,8 +94,8 @@ Looper::initialize (unsigned int index, unsigned int chan_count, float loopsecs,
 	_chan_count = chan_count;
 	
 	_ok = false;
-	requested_cmd = 0;
-	last_requested_cmd = 0;
+	requested_cmd = -1;
+	last_requested_cmd = -1;
 	request_pending = false;
 	_input_ports = 0;
 	_output_ports = 0;
@@ -181,9 +181,11 @@ Looper::initialize (unsigned int index, unsigned int chan_count, float loopsecs,
 	memset (ports, 0, sizeof(float) * LASTPORT);
 
 	memset(_down_stamps, 0, sizeof(nframes_t) * (Event::LAST_COMMAND+1));
+        /*
 	for (int i=0; i < (int) Event::LAST_COMMAND+1; ++i) {
 	  _down_stamps[i] = 1 << 31;
 	}
+        */
 
 	_longpress_frames = (nframes_t) lrint (srate * 1.0); // more than 1 secs is SUS
 	_doubletap_frames = (nframes_t) lrint (srate * 0.5); // less than 0.5 sec is double tap
@@ -446,12 +448,12 @@ Looper::set_replace_quantized(bool flag)
 }
 
 void 
-Looper::set_soloed (int index, bool value)
+Looper::set_soloed (int index, bool value, bool retrigger)
 {
 	if (index != (int) _index) {
 		// someone else is being soloed (or unsoloed), note our mute state, then mute self
 		if (value) {
-			if (ports[State] != LooperStateMuted) {
+			if (ports[State] != LooperStateMuted || ports[State] != LooperStateOffMuted) {
 				_pre_solo_muted = false;
 				Event ev;
 				ev.Type = Event::type_cmd_hit;
@@ -474,13 +476,19 @@ Looper::set_soloed (int index, bool value)
 		}
 	}
 	else {
+                // we are the target of the solo
 		_is_soloed = value;
 			
-		if (value && ports[State] == LooperStateMuted) {
+		if (value && ports[State] == LooperStateMuted) {                     
 			// ensure we are not muted if we are soloed
 			Event ev;
 			ev.Type = Event::type_cmd_hit;
-			ev.Command = Event::MUTE_OFF;
+                        if (retrigger) {
+                                ev.Command = Event::TRIGGER;
+                        }
+                        else {
+                                ev.Command = Event::MUTE_OFF;
+                        }
 			do_event(&ev);
 		}
 	}
@@ -639,6 +647,11 @@ Looper::recompute_latencies()
 	//cerr << "output lat: " << ports[OutputLatency] << endl;
 }
 
+bool Looper::has_loop() const
+{
+	return (_instances && _instances[0] && sl_has_loop(_instances[0]));
+}
+
 float
 Looper::get_control_value (Event::control_t ctrl)
 {
@@ -729,30 +742,30 @@ void Looper::set_port (ControlPort n, float val)
 {
 	switch (n)
 	{
-	case DryLevel:
-		_target_dry = val;
-		break;
-	case RelativeSync:
-		_relative_sync = val;
-		break;
-	case Event::ReplaceQuantized:
-		set_replace_quantized(val > 0.0f ? true : false);
-		break;
-	case TempoInput:
-		if (_tempo_stretch && ports[CycleLength] != 0.0f) {
-			// new ratio is origtempo/newtempo
-		        double tempo = (30.0 * ports[EighthPerCycleLoop] / ports[CycleLength]);
-			//cerr << "tempo calc: " << tempo << " tempo input: " << val << endl;
-			// clamp it if close to the same
-			tempo = (abs(tempo-val) < 0.001) ? val: tempo;
-			double newratio = min(4.0, max(0.5, tempo / (double) val)); 
-			_pending_stretch_ratio = newratio;
-			_pending_stretch = true;
-		}
-		// fallthrough intentional
-	default:
-		ports[n] = val;
-		break;
+		case DryLevel:
+			_target_dry = val;
+			break;
+		case RelativeSync:
+			_relative_sync = val;
+			break;
+		case Event::ReplaceQuantized:
+			set_replace_quantized(val > 0.0f ? true : false);
+			break;
+		case TempoInput:
+			if (_tempo_stretch && ports[CycleLength] != 0.0f) {
+				// new ratio is origtempo/newtempo
+				double tempo = (30.0 * ports[EighthPerCycleLoop] / ports[CycleLength]);
+				//cerr << "tempo calc: " << tempo << " tempo input: " << val << endl;
+				// clamp it if close to the same
+				tempo = (abs(tempo-val) < 0.001) ? val: tempo;
+				double newratio = min(4.0, max(0.5, tempo / (double) val)); 
+				_pending_stretch_ratio = newratio;
+				_pending_stretch = true;
+			}
+			// fallthrough intentional
+		default:
+			ports[n] = val;
+			break;
 	}
 }
 
@@ -762,7 +775,7 @@ Looper::is_longpress (int cmd)
 {
 	if ((int) cmd >= 0 && (int) cmd < (int) Event::LAST_COMMAND) {
 
-		return _running_frames > (_down_stamps[cmd] + _longpress_frames);
+		return (_down_stamps[cmd] > 0 && _running_frames > (_down_stamps[cmd] + _longpress_frames));
 	}
 	return false;
 }
@@ -775,10 +788,11 @@ Looper::do_event (Event *ev)
 		Event::command_t cmd = ev->Command;
 		requested_cmd = cmd;
 		request_pending = true;
-		
+		//fprintf(stderr, "Got HIT cmd: %d\n", cmd);
+
 		// a few special commands have double-tap logic
-		if (cmd == Event::RECORD_OR_OVERDUB || cmd == Event::RECORD_OR_OVERDUB_EXCL) {
-			if (_running_frames < (_down_stamps[cmd] + _doubletap_frames))
+		if (cmd == Event::RECORD_OR_OVERDUB || cmd == Event::RECORD_OR_OVERDUB_EXCL || cmd == Event::RECORD_OR_OVERDUB_SOLO) {
+			if (_down_stamps[cmd] > 0 && _running_frames < (_down_stamps[cmd] + _doubletap_frames))
 			{
 				// we actually need to undo twice!
 				requested_cmd = Event::UNDO_TWICE; 
@@ -793,9 +807,11 @@ Looper::do_event (Event *ev)
 			requested_cmd = cmd;
 			request_pending = true;
 
+			// fprintf(stderr, "Got DOWN cmd: %d\n", cmd);
+
 			// a few special commands have double-tap logic
-			if (cmd == Event::RECORD_OR_OVERDUB || cmd == Event::RECORD_OR_OVERDUB_EXCL) {
-				if (_running_frames < (_down_stamps[cmd] + _doubletap_frames))
+			if (cmd == Event::RECORD_OR_OVERDUB || cmd == Event::RECORD_OR_OVERDUB_EXCL || cmd == Event::RECORD_OR_OVERDUB_SOLO) {
+				if (_down_stamps[cmd] > 0 && _running_frames < (_down_stamps[cmd] + _doubletap_frames))
 				{
 					// we actually need to undo twice!
 					requested_cmd = Event::UNDO_TWICE; 
@@ -813,29 +829,29 @@ Looper::do_event (Event *ev)
 		if ((int) cmd >= 0 && (int) cmd < (int) Event::LAST_COMMAND)
 		{
 
-		  if (ports[State] != LooperStatePlaying
-			    || cmd == Event::REVERSE || cmd == Event::DELAY || cmd == Event::UNDO || cmd == Event::REDO)
+			if (ports[State] != LooperStatePlaying
+					|| cmd == Event::REVERSE || cmd == Event::DELAY || cmd == Event::UNDO || cmd == Event::REDO)
 			{
-				
+
 				if (ev->Type == Event::type_cmd_upforce) {
 					// special case if current state is mult or insert
 					// and the cmd is mult or insert and we're not quantized
 					// a SUS action here really means an unrounded action
 					if (ports[Quantize] == 0.0f
-					    && ((ports[State] == LooperStateMultiplying && cmd == Event::MULTIPLY)
-						|| (ports[State] == LooperStateInserting && cmd == Event::INSERT)))
+							&& ((ports[State] == LooperStateMultiplying && cmd == Event::MULTIPLY)
+								|| (ports[State] == LooperStateInserting && cmd == Event::INSERT)))
 					{
 						// this really should be handled down in the plugin
 						cmd = Event::RECORD;
 					}
-					
+
 					//cerr << "force up" << endl;
-					
+
 					requested_cmd = cmd;
 					request_pending = true;
 
 				}
-				else if (_running_frames > (_down_stamps[cmd] + _longpress_frames))
+				else if (_down_stamps[cmd] > 0 && _running_frames > (_down_stamps[cmd] + _longpress_frames))
 				{
 					//cerr << "long up" << endl;
 					requested_cmd = cmd;
@@ -848,20 +864,22 @@ Looper::do_event (Event *ev)
 					else if (cmd == Event::REDO) {
 						requested_cmd = Event::REDO_ALL;
 					}
-					else if (cmd == Event::RECORD_OR_OVERDUB) {
+					else if (cmd == Event::RECORD_OR_OVERDUB || cmd == Event::RECORD_OR_OVERDUB_EXCL || cmd == Event::RECORD_OR_OVERDUB_SOLO || cmd == Event::RECORD_OVERDUB_END_SOLO) {
 						// longpress of this turns into undo all for one-button goodness
 						requested_cmd = Event::UNDO_ALL;
 					}
 				}
 			}
-			
+			//fprintf(stderr, "Got UP cmd: %d  req: %d\n", cmd, requested_cmd);
+
+
 			_down_stamps[cmd] = 0;
 		}
 	}
 	else if (ev->Type == Event::type_control_change)
 	{
 		// todo: specially handle TriggerThreshold to work across all channels
-		
+
 		if ((int)ev->Control >= (int)Event::TriggerThreshold && (int)ev->Control < (int) Event::State) {
 
 			if (ev->Control == Event::Rate) {
@@ -1000,11 +1018,12 @@ Looper::run (nframes_t offset, nframes_t nframes)
 		if (ports[Multi] == requested_cmd) {
 			/* defer till next call */
 			ports[Multi] = -1;
+                        //fprintf(stderr,"Deferred to next run\n");
 		} else {
 			ports[Multi] = requested_cmd;
 			request_pending = false;
-			// cerr << "requested mode " << requested_cmd << endl;
-
+                        //fprintf(stderr,"Requested mode: %d\n", requested_cmd);
+                         
 			if (requested_cmd == Event::RECORD && ports[State] != LooperStateRecording) {
 				// record cmd, lets reset stretch and pitch ratios to 1 always
 				_pending_stretch_ratio = _stretch_ratio = 1.0;
@@ -1016,6 +1035,7 @@ Looper::run (nframes_t offset, nframes_t nframes)
 
 	} else if (ports[Multi] >= 0) {
 		ports[Multi] = -1;
+                //fprintf(stderr,"Reset to -1\n");
 		//cerr << "reset to -1\n";
 	}
 
@@ -1336,7 +1356,7 @@ Looper::run_loops (nframes_t offset, nframes_t nframes)
 	}
 	else 
 	{
-		// nornal operation
+		// normal operation
 
 		for (unsigned int i=0; i < _chan_count; ++i)
 		{
@@ -1362,10 +1382,6 @@ Looper::run_loops (nframes_t offset, nframes_t nframes)
 	for (unsigned int i=0; i < _chan_count; ++i)
 	{
 
-		if (_panner && _use_common_outs) {
-			// mix this output into common outputs
-			(*_panner)[i]->distribute (outbufs[i], com_obufs, 1.0f, nframes);
-		} 
 
 		if (_have_discrete_io && real_inbufs[i]) {
 			// just mix the dry into the outputs
@@ -1378,6 +1394,11 @@ Looper::run_loops (nframes_t offset, nframes_t nframes)
 			}
 		}
 
+		if (_panner && _use_common_outs) {
+			// mix this output into common outputs
+			(*_panner)[i]->distribute (outbufs[i], com_obufs, 1.0f, nframes);
+		} 
+
 		// calculate output peak post mixing with dry
 		compute_peak (outbufs[i], nframes, _output_peak);
 		
@@ -1685,7 +1706,7 @@ Looper::save_loop (string fname, LoopFileEvent::FileFormat format)
 
 		if (nframes == 0) {
 			// we're done, it shorted us somehow
-			cerr << "shorted" << endl;
+			//cerr << "shorted" << endl;
 			break;
 		}
 		
@@ -1921,18 +1942,23 @@ Looper::set_state (const XMLNode& node)
 
 	// load audio if we should
 	if ((prop = node.property ("loop_audio")) != 0) {
-	   ports[State] = LooperStatePaused; // force this
-	   if (!load_loop(prop->value())) {
-		   // use the basename of the filename with the path of the session file
-		   string filename = prop->value();
+		ports[State] = LooperStatePaused; // force this
+		if (!load_loop(prop->value())) {
+			// use the filename with the path of the session file
+			string filename = prop->value().c_str();
 
-		   if ((prop = node.property("session_filename")) != 0) {
-			   string sessfilename = prop->value();
-			   string newfilename = string(::dirname((char *)sessfilename.c_str())) + string("/") + string(::basename((char *)filename.c_str()));
+			if ((prop = node.property("session_filename")) != 0) {
+				//explicitly make a copy as dirname modifies it's input
+				const char * sessfilename = prop->value().c_str();
+				char * modifiable_copy = (char *)malloc(strlen(sessfilename) + 1);
+				strcpy(modifiable_copy, sessfilename);
+				char * directory = dirname(modifiable_copy);
+				string newfilename = string(directory) + string("/") + filename;
 
-			   load_loop(newfilename);
-		   }
-	   }
+				load_loop(newfilename);
+				free (modifiable_copy);
+			}
+		}
 	}
 
 	return 0;
