@@ -103,7 +103,8 @@ enum LooperState
 	LooperStateScratching,
 	LooperStateOneShot,
 	LooperStateSubstitute,
-	LooperStatePaused
+	LooperStatePaused,
+	LooperStateOffMuted = 20
 };
 
 	
@@ -395,4 +396,6 @@ extern void sl_set_replace_quantized (LADSPA_Handle instance, bool value);
 extern bool sl_get_replace_quantized (LADSPA_Handle instance);
 extern void sl_set_loop_index (LADSPA_Handle instance, unsigned int index, unsigned int chan);
 
+extern bool sl_has_loop (const LADSPA_Handle instance);
+
 #endif
