@@ -34,6 +34,7 @@
 #include "event_nonrt.hpp"
 
 #include "midi_bridge.hpp"
+#include "command_map.hpp"
 #include <midi++/port_request.h>
 
 // #if WITH_ALSA
@@ -119,7 +120,7 @@ static void usage(char *argv0)
 	fprintf(stderr, "  -L <pathname> , --load-session=<pathname> load initial session from pathname\n");
 	fprintf(stderr, "  -D <yes/no>, --discrete-io=[yes]  initial loops should have discrete input and output ports (default yes)\n");
 	fprintf(stderr, "  -p <num> , --osc-port=<num>  udp port number for OSC server (default is %d)\n", DEFAULT_OSC_PORT);
-	fprintf(stderr, "  -j <str> , --jack-name=<str> jack client name, default is sooperlooper_1\n");
+	fprintf(stderr, "  -j <str> , --jack-name=<str> jack client name, default is sooperlooper\n");
 	fprintf(stderr, "  -S <str> , --jack-server-name=<str> specify jack server name\n");
 	fprintf(stderr, "  -m <str> , --load-midi-binding=<str> loads midi binding from file or preset\n");
 	fprintf(stderr, "  -q , --quiet                 do not output status to stderr\n");
@@ -276,7 +277,7 @@ static void setup_signals()
 int main(int argc, char** argv)
 {
 	OptionInfo option_info;
-
+	CommandMap & cmdmap = CommandMap::instance();
 	//setup_signals();
 	
 	
@@ -370,10 +371,11 @@ int main(int argc, char** argv)
 #endif
 
 	if (midibridge && midibridge->is_ok()) {
-		if (!option_info.bindfile.empty()) {
-			midibridge->bindings().load_bindings (option_info.bindfile);
-		}
 		engine->set_midi_bridge(midibridge);
+		if (!option_info.bindfile.empty()) {
+			engine->load_midi_bindings(option_info.bindfile, false, cmdmap);
+			//midibridge->bindings().load_bindings (option_info.bindfile);
+		}
 	}
 	
 	// go into engine's non-rt event loop
